@@ -5,6 +5,7 @@
 .First.lib <- function(libname, pkgname) {
   #Load the necessary packages
   library(spc)
+  library(maptools)
 
   #Load the CIdata thing
   data(CIdata, package=pkgname)
@@ -14,6 +15,9 @@
   #Load the pre-evaluated Hypergeometric function for computing Anscombe residuals
   surveillance.gvar.hyp <- scan(file.path(.path.package('surveillance'),'data',"hypGeomSmall.txt"),quiet=TRUE)
   surveillance.gvar.z <- - c(0:1000/100, 11:100)
+
+  #Load the C code library for the glr stuff
+  library.dynam("surveillance", pkgname, libname)
 }
 
 
